import 'package:flutter/material.dart';
import 'package:flutter/foundation.dart';
import 'package:iroute/iroute.dart';
import 'iroute_state.dart';
import 'iroute_common.dart';

class IRoute extends RouterDelegate<String>
    with ChangeNotifier, PopNavigatorRouterDelegateMixin {
  final GlobalKey<NavigatorState> navigatorKey;

  PageList? pageList;

  IRouteState iRouteState = IRouteState();

  IRoute({this.pageList, required Map<String, RouteBuilder> routes})
      : navigatorKey = GlobalKey<NavigatorState>() {
    //在这里初始化routeState，否则必须在main中执行。
    if (!IRouteState.inited) IRouteState.init(routes);
    iRouteState.addListener(notifyListeners);
  }

  List<Page> _pages = [];

  Map<String, RouteBuilder> get routes => IRouteState().routes;

  static IRoute of(BuildContext context) {
    final delegate = Router.of(context).routerDelegate;
    assert(delegate is IRoute, 'Delegate type must match');
    return delegate as IRoute;
  }

  //如果指定了child
  //如果没有，则pages
  @override
  Widget build(BuildContext context) {
    //_pages不变化，则不显示
    //由此，直接赋予iRouteState.stack，则页面不会更新。
    //nested可以更新，是由于整个appShell重绘。重绘的时候，可能出现错误url进入某页面。
    _pages = iRouteState.stack.toList();

    //print('iRoute:build , url=$currentConfiguration');
    return Navigator(
        key: navigatorKey,
        pages: pageList != null
            ? [
                MaterialPage(
                  key: ValueKey('AppShell'), //UniqueKey(),
                  child: IRouteAppShell(
                    pageList: this.pageList!,
                    routes: this.routes,
                    iRouteState: iRouteState,
                  ),
                ),
              ]
            : _pages,
        onPopPage: (route, result) {
          if (!route.didPop(result)) {
            return false;
          }
          if (canPop) {
            pop();
          }
          return true;
        });
  }

  @override
  String? get currentConfiguration {
    return iRouteState.currentConfiguration;
  }

  @override
  dispose() {
    iRouteState.removeListener(notifyListeners);
    super.dispose();
  }

  push(String path) {
    //print('IRoute: push 准备push:$path');
    //if (path == currentConfiguration) return;
    iRouteState.push(path);

    //无需如此:
    //notifyListeners();
  }

  pop() {
    //print('iroute pop');
    iRouteState.pop();
    //notifyListeners();
  }

  bool get canPop => iRouteState.canPop;
  bool initing = false;

  // @override
  // Future<void> setInitialRoutePath(String configuration) {
  //   initing = true;
  //   return super.setInitialRoutePath(configuration);
  // }

  //setNewPath不需要notifyListeners()
  //注意：两种情况会执行这里
  //首先是setInitialRoutePath之后，我们可设置initing字段为true，而这里将其改为false
  //其次是响应浏览器地址栏变化，包括前进、后退、选历史记录、修改浏览器地址，此时initing=false
  //此时可注意，不向浏览器发送历史记录，如Router.neglect
  @override
  Future<void> setNewRoutePath(String path) async {
    //print('setNewPath:path=$path,stack=${iRouteState.stack.toString()}');

    iRouteState.setNewRoutePath(path); //加入path，且通知
    //return SynchronousFuture(null);
  }
}
