import 'iroute_nested.dart';
import 'iroute_state.dart';
import 'page_list.dart';
import 'package:flutter/material.dart';
import 'iroute_common.dart';

class IRouteAppShell extends StatefulWidget {
  final IRouteState iRouteState;
  final PageList pageList;
  final Map<String, RouteBuilder> routes;

  IRouteAppShell(
      {required this.iRouteState,
      required this.pageList,
      required this.routes});

  @override
  IRouteAppShellState createState() => IRouteAppShellState(
      pageList: pageList, routes: routes, iRouteState: iRouteState);
}

class IRouteAppShellState extends State {
  IRouteAppShellState(
      {required this.pageList,
      required this.routes,
      required this.iRouteState});

  final PageList pageList;
  final Map<String, RouteBuilder> routes;
  final IRouteState iRouteState;
  late IRouteNested _routerDelegate =
      IRouteNested(routes: routes, pageList: pageList);
  ChildBackButtonDispatcher? _backButtonDispatcher;

  List<BottomNavigationBarItem> items = [];

  @override
  void initState() {
    //print('AppShell:initState');
    //必须，否则后退功能不会刷新底部导航按钮位置
    //_routerDelegate.addListener(() => setState(() {}));
    super.initState();

    pageList.pages.forEach((element) {
      items.add(element.bottom);
    });
  }

  @override
  void dispose() {
    //_routerDelegate.removeListener(() => setState(() {}));
    super.dispose();
  }

  @override
  void didUpdateWidget(covariant IRouteAppShell oldWidget) {
    //_routerDelegate.iRouteState = iRouteState;
    super.didUpdateWidget(oldWidget);
  }

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    _backButtonDispatcher = Router.of(context)
        .backButtonDispatcher!
        .createChildBackButtonDispatcher();
  }

  @override
  Widget build(BuildContext context) {
    _backButtonDispatcher!.takePriority();
    //print(
    //    'AppShell:build,iRouteState.currentConfiguration=${iRouteState.currentConfiguration}');
    pageList.changeToUrl(iRouteState.currentConfiguration);

    return Scaffold(
      body: Router(
        routerDelegate: _routerDelegate,
        backButtonDispatcher: _backButtonDispatcher,
      ),
      bottomNavigationBar: BottomNavigationBar(
        type: BottomNavigationBarType.fixed,
        items: items,
        currentIndex: pageList.current, //_routerDelegate.selectedPage,
        onTap: (newIndex) {
          //print('点击按钮: ${newIndex == 1 ? "设置" : "主页"}');
          //print('pageList的当前url为${pageList.url}');
          _routerDelegate.selectedPage = newIndex;
          //print('push后：pageList的当前url为${pageList.url}');
          //print('${iRouteState.stack}');
          //print('pageList当前url为${pageList!.url}');
          //_routerDelegate.notifyListeners();
        },
      ),
    );
  }
}
