import 'package:flutter/material.dart';
import 'package:flutter/foundation.dart';
import 'iroute_state.dart';
import 'page_list.dart';
import 'iroute_common.dart';

class IRouteNested extends RouterDelegate<String>
    with ChangeNotifier, PopNavigatorRouterDelegateMixin<String> {
  GlobalKey<NavigatorState> get navigatorKey =>
      GlobalObjectKey<NavigatorState>(this);

  static IRouteNested of(BuildContext context) {
    final delegate = Router.of(context).routerDelegate;
    assert(delegate is IRouteNested, 'Delegate type must match');
    return delegate as IRouteNested;
  }

  IRouteState iRouteState = IRouteState(); //stack是单例的，全局只有一个
  late Map<String, RouteBuilder> routes;

  late PageList pageList;

  //error:Null check operator used on a null value
  int get selectedPage {
    return pageList.current;
  }

  set selectedPage(int value) {
    pageList.current = value;
    //print(
    //    'nested: selectPage,url in pageList is ${pageList.pages[pageList.current].currentUrl}');
    push(pageList.pages[pageList.current].currentUrl);
  }

  @override
  void dispose() {
    super.dispose();
    //iRouteState.root.removeListener(this.notifyListeners);
  }

  IRouteNested({required this.routes, required this.pageList}) {
    //if (stack.root == null) throw ('static should have a root');
    //stack.root.addListener(this.notifyListeners);
    if (routes.containsKey('_')) {
      addHome = (context, params) => routes['_']!(context, params);
    }
  }

  RouteBuilder? addHome;
  bool get canPop {
    return iRouteState.canPop;
  }

  @override
  Widget build(BuildContext context) {
    //print('nested:build, 当前url为${iRouteState.currentConfiguration}');
    pageList.changeToUrl(iRouteState.currentConfiguration);
    //print(
    //'nested:build, pageList.current为${pageList.current},url=${pageList.url}');

    List<Page> _pages = [iRouteState.stack.last];
    // if (pageList.url != iRouteState.currentConfiguration) {
    //   print('nested build:当前页面的路由，和全局当前路由不同！');
    // }

    return Navigator(
        key: navigatorKey,
        pages: _pages,
        onPopPage: (route, result) {
          if (!route.didPop(result)) {
            return false;
          }
          if (canPop) {
            pop();
          }
          return true;
        });
  }

  @override
  Future<bool> popRoute() {
    //print('Pop Route');
    return super.popRoute();
  }

  void push(String path) {
    //print(
    //    '进入nested push,准备push $path,当前url是${iRouteState.currentConfiguration}');
    String url = path;

    if (!path.contains(':ipage') && path != '/') {
      url = url + ':ipage=${pageList.current}';
    }
    //print('准备push:$url');
    pageList.pages[pageList.current].currentUrl = url;
    iRouteState.push(url);
  }

  void pop() {
    if (!canPop) return;
    //print('pop in root');
    iRouteState.pop(); //这里有重绘，后面的是否会执行？
  }

  @override
  Future<void> setNewRoutePath(String path) async {
    // This is not required for inner router delegate because it does not
    // parse route
    assert(false);
  }
}
